@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2004 Edscott Wilson Garcia
+ * Copyright (C) 2002-2003 Edscott Wilson Garcia
  * EMail: edscott@imp.mx
  *
  *
@@ -22,18 +22,36 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
 
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
+#endif
 #include <errno.h>
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
+#endif
+#ifdef HAVE_MEMORY_H
 #include <memory.h>
+#endif
+#ifdef HAVE_REGEX_H
 #include <regex.h>
+#endif
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <glob.h>
 #include <dbh.h>
@@ -41,23 +59,19 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include <libxfce4util/util.h>
 
 #include "glade_support.h"
 
 #include "constants.h"
 #include "types.h"
 
-#include "basic_row.h"
-
-#include "bookmarks.h"
-
-#include "dnd.h"
+#include "add_file.h"
+#include "add_node_contents.h"
+#include "dummies.h"
 #include "entry.h"
-
-#ifdef USE_FILTER_BAR
 #include "filter.h"
-#endif
-
+#include "gdir.h"
 #include "misc.h"
 #include "refresh.h"
 #include "widgets.h"
@@ -65,8 +79,7 @@
 
 /*#define DEBUG*/
 
-extern gchar *bookfile;
-extern root_t root[];
+extern int stop;
 
 static DBHashTable *bookmarks = NULL;
 static DBHashTable *newbookmarks = NULL;
@@ -75,34 +88,25 @@ static GtkTreeView *target_treeview;
 static int bookmarks_count;
 static unsigned int smallcount, countbyte;
 static int target_type;
-
-#ifdef USE_FILTER_BAR
-static const regex_t *target_preg;
-#endif
-
-static xfdir_t bm_xfdir;
+static regex_t *target_preg;
+static gdir_t bm_gdir;
 
 static GList *d_list = NULL;
 static int check_dir(char *path);
 
-G_MODULE_EXPORT
-void set_book_combo (void)
+extern gchar *bookfile;
+extern root_t root[];
+void set_book_combo(GtkTreeView * treeview)
 {
     GList *tmp,*new_list = NULL;
     /*GtkEntry *entry;*/
-    GtkWidget *w = WIDGET("input_combo");
-	gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
+    GtkWidget *w = lookup_widget((GtkWidget *)treeview, "input_combo");
 	glob_t dirlist;
 	int i;
-	gchar *globstring,*p;
-
-#if 0
+    	char bm_dir[_POSIX_PATH_MAX];
+	gchar *globstring;
+        xfce_get_userfile_r(bm_dir, _POSIX_PATH_MAX-1, "xffm");
 	globstring = g_strconcat(bm_dir,"/*.bm.dbh",NULL);
-#endif
-	p = g_build_filename(xdg_dir,BOOKMARK_DIR,NULL);
-	g_free(xdg_dir);
-	globstring = g_strconcat(p,G_DIR_SEPARATOR_S,"*.bm.dbh",NULL);
-	g_free(p);
 	
 	if (glob (globstring, GLOB_ERR, NULL, &dirlist) != 0) {
 		return;
@@ -117,6 +121,8 @@ void set_book_combo (void)
 	}
   	globfree (&dirlist);
 
+    /*entry = (GtkEntry *) lookup_widget((GtkWidget *)treeview, "input_entry");
+    gtk_entry_set_text(entry, (gchar *)new_list->data);*/
     
     gtk_combo_set_popdown_strings(GTK_COMBO(w), new_list);
     for (tmp=new_list;tmp;tmp=tmp->next){
@@ -127,48 +133,40 @@ void set_book_combo (void)
     g_list_free(new_list);
 }
 
-G_MODULE_EXPORT
-void reload_book (GtkTreeView *treeview){
+void reload_book(GtkTreeView *treeview){
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     GtkTreePath *treepath;
     tree_entry_t *en;
     GtkTreeIter iter;
     get_bookmark_root(treeview,&iter, &en);
-    clear_dnd_selection_list();
     treepath = gtk_tree_model_get_path(treemodel, &iter);
+    clear_dnd_selection_list();
     gtk_tree_selection_select_path (selection,treepath);
     gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
     gtk_tree_path_free(treepath);
-
-    update_text_cell_for_row(NAME_COLUMN,treemodel, &iter,(bookfile)?
-	    g_strdup(my_utf_string(bookfile)):
-	    g_strdup(my_utf_string(_(root[ROOT_BOOKMARKS].label)))
-	    );
+    gtk_tree_store_set((GtkTreeStore *) treemodel, &iter, 
+			NAME_COLUMN, 
+			(bookfile)?my_utf_string(bookfile):
+			my_utf_string(_(root[ROOT_BOOKMARKS].label)),
+		       	-1);
     on_refresh((GtkButton *) treeview,NULL);
 }
 
 
-G_MODULE_EXPORT
-char *get_bookfile_path (void){
-    gchar *bookmarks_dir;
-    static gchar *bookmarks_path=NULL;
-    gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
-
-    bookmarks_dir = g_build_filename(xdg_dir,BOOKMARK_DIR,NULL);    
-    g_free(xdg_dir);
-    if(!check_dir(bookmarks_dir)) {
-	    g_free(bookmarks_dir);
-	    return NULL;
-    }
-    if (bookmarks_path)  g_free(bookmarks_path);
-    
+char *get_bookfile_path(void){
+    static char bookmarks_path[_POSIX_PATH_MAX];
+    gchar *g;
+    xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1, "xffm");
+    if(!check_dir(bookmarks_path)) return NULL;
     if (bookfile) {
-	    bookmarks_path=g_strconcat(bookmarks_dir,G_DIR_SEPARATOR_S,bookfile,".bm.dbh",NULL);
-    } else {
-	    bookmarks_path=g_strconcat(bookmarks_dir,G_DIR_SEPARATOR_S,"bookmarks.dbh",NULL);
-    }
-    g_free(bookmarks_dir);
+	    g=g_strconcat("xffm/",bookfile,".bm.dbh",NULL);
+	    xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1,
+		    g);
+	    g_free(g);
+	    g=NULL;
+    } else xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1,
+		    "xffm%cbookmarks.dbh", G_DIR_SEPARATOR);
 #ifdef DEBUG
     printf("DBG: using %s\n",bookmarks_path);
 #endif
@@ -188,8 +186,7 @@ static void bookmarks_dlist(DBHashTable * dbh)
 }
 
 
-G_MODULE_EXPORT
-GList *get_bookmark_dirlist (void)
+GList *get_bookmark_dirlist(void)
 {
     char *bookmarks_path=get_bookfile_path();
     d_list = NULL;
@@ -238,7 +235,7 @@ static void add2treeview(GtkTreeView * treeview, char *path)
     {
 	name = strrchr(path, '/');
 	if(!name)
-	    assert_not_reached();
+	    g_assert_not_reached();
 	name++;
     }
     get_bookmark_root(treeview, &target, &en);
@@ -252,8 +249,8 @@ static void add2treeview(GtkTreeView * treeview, char *path)
           en=mk_net_entry(path,en->type);
     }
     else en = stat_entry(path, en->type);
-    add_row(treemodel, &target, NULL, NULL,  en, name);
-    erase_dummy_row(treemodel, &target,NULL);
+    prepend_file(treeview, &target, en, name);
+    erase_dummy(treeview, &target);
 
     /* expand + scroll to */
     treepath = gtk_tree_model_get_path(treemodel, &target);
@@ -266,8 +263,7 @@ static void add2treeview(GtkTreeView * treeview, char *path)
     return;
 }
 
-G_MODULE_EXPORT
-int add2bookmarks (GtkTreeView * treeview, char *path)
+int add2bookmarks(GtkTreeView * treeview, char *path)
 {
     char *bm=get_bookfile_path();
     struct stat st;
@@ -277,8 +273,8 @@ int add2bookmarks (GtkTreeView * treeview, char *path)
     GtkTreePath *treepath;
     gboolean is_net_stuff=FALSE;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    tree_details_t *tree_details = get_tree_details(treeview);
     gchar *d_path=NULL;
-    gchar *message=NULL;
 
     if (strncmp(path,"smb://",strlen("smb://"))==0||
 	strncmp(path,"SMB://",strlen("SMB://"))==0) {
@@ -319,37 +315,28 @@ int add2bookmarks (GtkTreeView * treeview, char *path)
     
     if(!DBH_load(bookmarks))
     {
+
 	memcpy(DBH_DATA(bookmarks), (void *)path, strlen(path) + 1);
 	DBH_set_recordsize(bookmarks, strlen(path) + 1);
 	if(!DBH_update(bookmarks))
 	{
-	    const gchar *fmt=N_("\%s NOT booked (\%s)");
-	    message=g_strdup_printf(fmt,d_path,DBH_KEY(bookmarks));
-	    print_diagnostics("xfce/error", message, "\n", NULL);
-	    g_free(message);
+	    print_diagnostics(treeview, "xf_ERROR_ICON", d_path, " ", _("NOT booked"), " (", DBH_KEY(bookmarks), ")\n", NULL);
 	}
 	else
 	{
-	    const gchar *fmt=N_("\%s booked");
 	    add2treeview(treeview, path);
-	    message=g_strdup_printf(fmt,d_path);
-	    print_diagnostics("xfce/error", message, "\n", NULL);
-	    g_free(message);
-	    if(d_path && strlen(d_path) >= 2){
-		message=g_strdup_printf(fmt,abreviate(d_path));
-		print_status("xfce/info",message, NULL);
-		g_free(message);
-	    }
+	    print_diagnostics(treeview, "xf_INFO_ICON", d_path, " ", _("booked"), "\n", NULL);
+	    if(d_path && strlen(d_path) >= 2)
+		print_status(treeview, "xf_INFO_ICON", __VY_FN__(d_path), " ", _("booked"), NULL);
+
 	}
 	DBH_close(bookmarks);
-	unset_load_wait();
+	unset_load_wait(&tree_details);
 	g_free(d_path);
 	d_path=NULL;
 	return TRUE;
     }
-    message=g_strdup_printf(_("\%s already in book"),path);
-    print_diagnostics("xfce/warning",message, "\n", NULL);
-    g_free(message);
+    print_diagnostics(treeview, "xf_WARNING_ICON", path, " ", _("already in book"), "\n", NULL);
     DBH_close(bookmarks);
     g_free(d_path);
     d_path=NULL;
@@ -378,60 +365,52 @@ static void add_bookmarks(DBHashTable * dbh)
     /*printf("DBG:%s -> %s\n",fullpath,p); */
     if(!p || strlen(p) <= 1) return;
     p++;
-    
+    if(regexec(target_preg, (const char *)p, 0, NULL, 0))
+	return;
 
     if(stat(fullpath, &st) >= 0){
-    	bm_xfdir.gl[bm_xfdir.pathc].en = stat_entry(fullpath, target_type);
+    	bm_gdir.gl[bm_gdir.pathc].en = stat_entry(fullpath, target_type);
     } 
     else if (strncmp(fullpath,"smb://",strlen("smb://"))==0||
 	strncmp(fullpath,"SMB://",strlen("SMB://"))==0){
-          bm_xfdir.gl[bm_xfdir.pathc].en=mk_net_entry(fullpath,target_type);
+          bm_gdir.gl[bm_gdir.pathc].en=mk_net_entry(fullpath,target_type);
     } else return;
-    if(!bm_xfdir.gl[bm_xfdir.pathc].en) assert_not_reached();
-    
-    SET_TOP_BOOKMARK(bm_xfdir.gl[bm_xfdir.pathc].en->subtype);
+    if(!bm_gdir.gl[bm_gdir.pathc].en) g_assert_not_reached();
          
-    bm_xfdir.gl[bm_xfdir.pathc].pathv = g_strdup(p);
-    bm_xfdir.pathc++;
+    bm_gdir.gl[bm_gdir.pathc].pathv = g_strdup(p);
+    bm_gdir.pathc++;
     return;
 }
 
 
-G_MODULE_EXPORT
-int open_bookmarks (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data)
+int open_bookmarks(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data)
 {
     char *bookmarks_path=get_bookfile_path();
-    tree_entry_t *en;
-    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    tree_entry_t *en = get_entry(treeview, iter);
+    tree_details_t *tree_details = get_tree_details(treeview);
     if (!bookmarks_path) return -1;
 
-    gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
-    if (!en) return -1;   
     target_iter = iter;
     target_treeview = treeview;
     target_type = en->type;
  
     smallcount = 0, countbyte = 0x10;	/* for dummy tag */
     bookmarks_count = 0;	/* for count step */
-    bm_xfdir.pathc = 0;		/* for read step */
+    bm_gdir.pathc = 0;		/* for read step */
     chmod(bookmarks_path,S_IRUSR|S_IWUSR);
     if((bookmarks = DBH_open(bookmarks_path)) == NULL)
 	return -1;
     /*printf("DBG:opening book\n"); */
     
-    cursor_wait();
+    cursor_wait(treeview);
 
-#ifdef USE_FILTER_BAR
 /* returns compiled regex and sets filter in tree_entry */
-    if (!en) target_preg = NULL;
-    else target_preg = compile_regex_filter(en->filter,SHOWS_HIDDEN(en->type));
-    /*target_preg = get_regex_filter(en);*/
-#endif
+    target_preg = get_regex_filter(treeview, en);
 
     /*printf("DBG:counting book elements\n"); */
 
     DBH_foreach_sweep(bookmarks, count_bookmarks);
-    /*printf("DBG:pathc=%d\n",bm_xfdir.pathc); */
+    /*printf("DBG:pathc=%d\n",bm_gdir.pathc); */
 
     if(DBH_ERASED_SPACE(bookmarks))
     {
@@ -442,47 +421,54 @@ int open_bookmarks (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * tr
 	UNSET_ERASED_SPACE(en->type);
     if(bookmarks_count)
     {
-	bm_xfdir.gl = (dir_t *) malloc(bookmarks_count * sizeof(dir_t));
-	if(!bm_xfdir.gl)
-	    assert_not_reached();
+	bm_gdir.gl = (dir_t *) malloc(bookmarks_count * sizeof(dir_t));
+	if(!bm_gdir.gl)
+	    g_assert_not_reached();
 	DBH_foreach_sweep(bookmarks, add_bookmarks);
 	/*printf("DBG:count=%d\n",bookmarks_count); */
-	if(bookmarks_count != bm_xfdir.pathc)
+	if(bookmarks_count != bm_gdir.pathc)
 	{
+	    /* printf("DBG: %s=%d, %s=%d", "listed", bm_gdir.pathc,
+			    "counted", bookmarks_count);*/
+	    /*print_diagnostics(treeview, "xf_WARNING_ICON", 
+			    _("book needs a purge"), "\n", 
+			    NULL);*/
 	    SET_ERASED_SPACE(en->type);
 	}
-	add_contents_row(treemodel, iter, &bm_xfdir);
-	xfdirfree(&bm_xfdir);
+	add_node_contents(treeview, iter, &bm_gdir);
+	gdirfree(&bm_gdir);
     }
     else
     {
 	SET_LOADED(en->type);
-	reset_dummy_row(treemodel, iter,NULL,en,"xfce/warning",_("Loading..."));
+	reset_dummy(treeview, iter, 0);
     }
     DBH_close(bookmarks);
 
-    if(tree_details->stop)
+    if(!en->tag)
+	en->tag = (char *)malloc(_POSIX_PATH_MAX);
+    if(stop)
     {
-	tree_details->stop = FALSE;
-	/*sprintf(en->tag, "%s : %s", FILENAME(en),strerror(ETIMEDOUT));*/
-	en->tag=g_strconcat(FILENAME(en)," : ",strerror(ETIMEDOUT),NULL);
+	stop = FALSE;
+	sprintf(en->tag, "%s : %s", FILENAME(en),
+		strerror(ETIMEDOUT));
     }
     else
     {
-	hide_stop();
-	/*sprintf(en->tag, "%s : %d %s", _("Book"), bm_xfdir.pathc, _("items"));*/
-	en->tag=g_strdup_printf ("%s : %d %s", _("Book"), bm_xfdir.pathc, _("items"));	
-	if (!bm_xfdir.pathc){
-	   /*char *bp=get_bookfile_path();*/
-	   reset_dummy_row(treemodel, iter,NULL,en,"xfce/info", _("Use drag+drop to add"));
+	hide_stop(tree_details->window);
+	sprintf(en->tag, "%s : %d %s", _("Book"), bm_gdir.pathc, _("items"));
+	if (!bm_gdir.pathc){
+	   char *bp=get_bookfile_path();
+	   reset_dummy(treeview, iter, 3);
 	   /* do an automatic purge:  NO!!!*/
 	   /*unlink(bp);*/
 	}
-	else erase_dummy_row(treemodel, iter,NULL);
 	
     }
 
-    cursor_reset();
+    /*gtk_widget_thaw_child_notify  ((GtkWidget *)treeview);*/
+    regfree(target_preg);
+    cursor_reset(treeview);
 
     return 0;
 }
@@ -490,15 +476,14 @@ int open_bookmarks (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * tr
 /* This is basically a DBH_regen(), except that temporary file is
  * within the scope of xffm, not dbh */
 
-G_MODULE_EXPORT
-void purge_bookmarks (DBHashTable * dbh)
+void purge_bookmarks(DBHashTable * dbh)
 {
     struct stat st;
     char *p;
     char *fullpath = (char *)DBH_DATA(dbh);
 
     if(!newbookmarks)
-	assert_not_reached();
+	g_assert_not_reached();
     p = strrchr(fullpath, '/');
     if(p)
     {
@@ -515,7 +500,7 @@ void purge_bookmarks (DBHashTable * dbh)
     DBH_set_recordsize(newbookmarks, DBH_RECORD_SIZE(bookmarks));
     /* write the record to the new file; */
     if(!DBH_update(newbookmarks))
-	assert_not_reached();
+	g_assert_not_reached();
     /*printf("DBG:clean record: %s\n",fullpath); */
     return;
 }
@@ -524,22 +509,21 @@ void purge_bookmarks (DBHashTable * dbh)
 
 /****  callbacks *********/
 
-
-G_MODULE_EXPORT
-void on_clear_all_bookmarks_activate (GtkMenuItem * menuitem, gpointer user_data)
+void on_clear_all_bookmarks_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
     char *bm=get_bookfile_path();
-    GtkTreeIter parent;
+    GtkTreeIter iter, parent;
     tree_entry_t *en;
-    gint tree_id = get_active_tree_id();
-    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
-    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
+    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
 
     get_bookmark_root(treeview, &parent, &en);
-    prune_row(treemodel, &parent, NULL, en);
-    insert_dummy_row(treemodel, &parent, NULL, en,"xfce/info",_("Use drag+drop to add"));
-    
-    
+    while(gtk_tree_model_iter_children(treemodel, &iter, &parent))
+    {
+	gtk_tree_store_remove((GtkTreeStore *) treemodel, &iter);
+    }
+    add_dummy(treeview, &parent);
+    reset_dummy(treeview, &parent, 3);
     unlink(bm);
     UNSET_ERASED_SPACE(en->type);
 
@@ -548,26 +532,19 @@ void on_clear_all_bookmarks_activate (GtkMenuItem * menuitem, gpointer user_data
 
 
 
-G_MODULE_EXPORT
-void on_purge_bookmarks1_activate (GtkMenuItem * menuitem, gpointer user_data)
+void on_purge_bookmarks1_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
-    gint tree_id = get_active_tree_id();
-    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
+    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+    tree_details_t *tree_details = get_tree_details(treeview);
     GtkTreeIter iter;
     tree_entry_t *en;
-    gchar *fname;
+    char fname[_POSIX_PATH_MAX];
     char *bookmarks_path=get_bookfile_path();
 
     if (!bookmarks_path) return;
-    {
-	gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
-	fname = g_build_filename(xdg_dir,BOOKMARK_DIR,NULL);
-	g_free(xdg_dir);
-    }
+    xfce_get_userfile_r(fname, _POSIX_PATH_MAX-1, "xffm");
     chdir(fname);
-    g_free(fname);
-    fname = g_strdup("bookmarks.XXXXXX");
-    /*strcpy(fname, "bookmarks.XXXXXX");*/
+    strcpy(fname, "bookmarks.XXXXXX");
     close(mkstemp(fname));
 
     chmod(bookmarks_path,S_IRUSR|S_IWUSR);
@@ -580,25 +557,22 @@ void on_purge_bookmarks1_activate (GtkMenuItem * menuitem, gpointer user_data)
     get_bookmark_root(treeview, &iter, &en);
     UNSET_ERASED_SPACE(en->type);
     on_refresh((GtkButton *) tree_details->window, NULL);
-    chdir(GETWD);
-    g_free(fname);
+
 }
 
-G_MODULE_EXPORT
-void on_remove_from_bookmarks_activate (GtkMenuItem * menuitem, gpointer user_data)
+void on_remove_from_bookmarks_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
     char *bm=get_bookfile_path();
     GtkTreeIter iter, parent;
     tree_entry_t *en;
-    gint tree_id = get_active_tree_id();
-    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
-    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
-    GtkTreeSelection *selection = tree_details->treestuff[tree_id].selection;
+    GtkTreeView *treeview = get_selected_treeview((GtkWidget *) menuitem);
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
 
     /*get selection */
-    en = get_selected_entry( &iter);
+    en = get_selected_entry(treeview, &iter);
     if(!en)
-	assert_not_reached();
+	g_assert_not_reached();
 
     chmod(bm,S_IRUSR|S_IWUSR);
     if((bookmarks = DBH_open(bm)) != NULL)
@@ -616,67 +590,26 @@ void on_remove_from_bookmarks_activate (GtkMenuItem * menuitem, gpointer user_da
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
     if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
     {
-	remove_row(treemodel, &iter,NULL,NULL);
+
+	gtk_tree_store_remove((GtkTreeStore *) treemodel, &iter);
     }
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
     get_bookmark_root(treeview, &parent, &en);
-    insert_dummy_row(treemodel, &parent,NULL,en,"xfce/info",_("Use drag+drop to add"));
+    if(!gtk_tree_model_iter_children(treemodel, &iter, &parent))
+    {
+	add_dummy(treeview, &parent);
+	reset_dummy(treeview, &parent, 3);
+    }
     SET_ERASED_SPACE(en->type);
 }
 
 
-G_MODULE_EXPORT
 void
 on_save_book                           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    show_input(SAVE_BOOK_INPUT);
-
-}
-
-G_MODULE_EXPORT
-void
-on_rememberbook_activate               (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-  	glob_t dirlist;
-	int i;
-    	gchar *bm_dir;
-	gchar *globstring;
-	gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
-
-	bm_dir = g_build_filename(xdg_dir,BOOKMARK_DIR,NULL);
-	g_free(xdg_dir);
-	
-	globstring = g_strconcat(bm_dir,G_DIR_SEPARATOR_S,"*.bm.dbh",NULL);
-
-	print_diagnostics("xfce/info",_("Book"),":\n",NULL);
-
-	
-	if (glob (globstring, GLOB_ERR, NULL, &dirlist) != 0) {
-	   print_diagnostics("xfce/warning","Nothing found",NULL);
-#ifdef DEBUG
-           printf ("DBG:%s: no match\n", globstring);
-#endif
-	   
-	} else {
-	   print_diagnostics("nonverbose",_("Files found=")," ",NULL);
-	   for (i = 0; i < dirlist.gl_pathc; i++){
-	      gchar *p,*g=g_path_get_basename(dirlist.gl_pathv[i]);
-	      if(!strstr(g,".bm.dbh")) continue;
-	      p=strstr(g,".bm.dbh");
-	      *p=0;
-	      print_diagnostics("nonverbose",g,(i == dirlist.gl_pathc-1)?
-			      "\n":", ",NULL);
-#ifdef DEBUG
-              /*printf ("DBG:glob match: %s\n", dirlist.gl_pathv[i]);*/
-              printf ("DBG:glob match: %s\n", g);
-	      
-#endif
-	      g_free(g);g=NULL;
-	   }
-	}
-  	globfree (&dirlist);
+    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+    show_input(treeview, SAVE_BOOK_INPUT);
 
 }
 
