@@ -361,6 +361,7 @@ int rsync(GtkTreeView * treeview, char **srcs, char *tgt)
 	      term = getenv("TERM");
 	    }
 	    g_free(c);
+	    c=NULL;
     }
 	      
     if(!term || strlen(term) == 0)
@@ -402,13 +403,19 @@ int rsync(GtkTreeView * treeview, char **srcs, char *tgt)
            print_diagnostics(treeview, "xf_ERROR_ICON",
 			  strerror(ENOENT),":", prog1,"\n", NULL);
 	   return FALSE;
-       } else g_free(test);
+       } else {
+	       g_free(test);
+	       test=NULL;
+       }
        test=g_find_program_in_path(prog2);
        if (!test) {
            print_diagnostics(treeview, "xf_ERROR_ICON",
 			  strerror(ENOENT),":", prog2,"\n", NULL);
 	   return FALSE;
-       } else g_free(test);
+       } else {
+	      g_free(test);
+	      test=NULL;
+       }
     }
        
 	       
@@ -498,8 +505,10 @@ char *randomTmpName(char *ext)
 {
     static char *fname = NULL;
     int fnamelen;
-    if(fname)
+    if(fname){
 	g_free(fname);
+	fname=NULL;
+    }
     if(ext == NULL)
 	fnamelen = strlen("/tmp/xffm.XXXXXX") + 1;
     else
@@ -518,6 +527,34 @@ char *randomTmpName(char *ext)
     return fname;
 }
 
+static FILE *plaintmpfile;
+static void get_selection(GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
+{
+    GtkTreeView *treeview = (GtkTreeView *) data;
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+    tree_entry_t *en;
+    gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
+    if (en && en->path) fprintf(plaintmpfile, "%d\t%s\t%s\n", TR_COPY, en->path, en->path);
+    return;
+}
+
+char *PlainTmpList(GtkTreeView *treeview,GtkTreeSelection *selection)
+{
+    char *fname = randomTmpName(NULL);
+
+    D(printf("DBG: fname=%s\n",(fname)?fname:"null");)
+    if(!fname) return NULL;
+    
+    if((plaintmpfile = fopen(fname, "w")) == NULL){
+    	D(printf("DBG: cannot open %s for write\n",(fname)?fname:"null");)
+	return NULL;
+    }
+    gtk_tree_selection_selected_foreach(selection, get_selection, (gpointer) treeview);
+    /*    fprintf(tmpfile, "%d\t%s\t%s\n", TR_COPY, src, tgt);*/
+    fclose(plaintmpfile);
+    return fname;
+}
+
 
 char *SimpleTmpList(char *tgt, char *src)
 {
@@ -540,8 +577,10 @@ char *mktgpath(char *target_dir, char *source)
     name = strrchr(source, '/');
     if(!name)
 	g_assert_not_reached();
-    if(target)
+    if(target){
 	g_free(target);
+	target=NULL;
+    }
     target = (char *)malloc(strlen(target_dir) + strlen(name) + 1);
     sprintf(target, "%s%s", target_dir, name);
     if(!target)
@@ -564,6 +603,7 @@ static void warn_target_exists(GtkTreeView * treeview, char *target)
 	dlg_result = DLG_CANCEL;
 	make_dialog(treeview, NULL, g);
 	g_free(g);
+	g=NULL;
     }
 }
 
@@ -732,6 +772,7 @@ char *CreateTmpList(GtkTreeView * treeview, GList * list, tree_entry_t * t_en)
 		    }
 		}
 		g_free(route);
+		route=NULL;
 	    }
 
 	}
@@ -833,6 +874,7 @@ gboolean DirectTransfer(GtkTreeView * treeview, int mode, char *tmpfile)
 	    if(len <= 0)
 	    {
 		g_free(src_lnk);
+		src_lnk=NULL;
 		switch (error_continue(treeview, strerror(errno), source))
 		{
 		    case DLG_CANCEL:
@@ -847,6 +889,7 @@ gboolean DirectTransfer(GtkTreeView * treeview, int mode, char *tmpfile)
 	    src_lnk[len] = 0;
 	    result = symlink(src_lnk, target);
 	    g_free(src_lnk);
+	    src_lnk=NULL;
 	    if(result >= 0 && (mode & TR_MOVE) && (unlink(source) < 0))
 	    {
 		print_diagnostics(treeview, "xf_ERROR_ICON", strerror(errno), " : ", source, "\n", NULL);
@@ -921,10 +964,11 @@ int recursive_count_files(char *path)
 	    g_assert_not_reached();
 	}
 	sprintf(fullpath, "%s/%s", path, dir->d_name);
-	lstat(fullpath, &st);
 	countT++;
-	smallcount++;
-	sizeT += st.st_size;
+	if (lstat(fullpath, &st) != -1) {
+	  smallcount++;
+	  sizeT += st.st_size;
+	}
 	if(S_ISDIR(st.st_mode))
 	    recursive_count_files(fullpath);
 	else if(smallcount & 0x100)
@@ -964,16 +1008,16 @@ static gint ParentCount(gpointer data)
 	type = atoi(strtok(line, "\t"));
 	source = strtok(NULL, "\t");
 	recursive_count_files(source);
-	lstat(source, &st);
 	countT++;
-	sizeT += st.st_size;
+	if (lstat(source, &st) != -1) {
+	  sizeT += st.st_size;
+	}
 	sprintf(line, "%s %s", _("Counted"), sizetag(sizeT, countT));
 	print_status(treeview, "xf_INFO_ICON", line, NULL);
     }
     fclose(tfile);
     g_free(line);
-
-
+    line=NULL;
     gtk_main_quit();
     return FALSE;
 }
@@ -1084,19 +1128,13 @@ void on_countfiles_activate(GtkMenuItem * menuitem, gpointer user_data)
     GtkTreeView *treeview = get_selected_treeview((GtkWidget *) menuitem);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    /*tree_details_t *tree_details=get_tree_details(treeview); */
-    tree_entry_t *en;
     GtkTreeIter iter;
 
-    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
-    if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
-    {
-	gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
-	strcpy(child_file, SimpleTmpList(en->path, en->path));
+	char *fname;
+	if ((fname=PlainTmpList(treeview,selection))==NULL) g_assert_not_reached();
+	strcpy(child_file, fname);
 	count_window(treeview);
 	unlink(child_file);
-    }
-    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
     /* count total files to transfer */
     return;
 }
@@ -1124,6 +1162,7 @@ static gboolean SubChildTransfer(char *target, char *source)
 		stat(route, &t_stat);
 	    }
 	    g_free(route);
+	    route=NULL;
 	}
     }
     /* do the link business before going recursive:
@@ -1160,9 +1199,11 @@ static gboolean SubChildTransfer(char *target, char *source)
 	if(symlink(src_lnk, target) < 0)
 	{
 	    g_free(src_lnk);
+	    src_lnk=NULL;
 	    return process_error("ln", RW_ERROR_WRITING_TGT);
 	}
 	g_free(src_lnk);
+	src_lnk=NULL;
 	fprintf(stdout, "child:item:%d\n", ++child_path_number);
 	if(child_mode & TR_MOVE)
 	{
@@ -1394,6 +1435,7 @@ static void ChildTransfer(gpointer data)
   cut_out:
     fclose(tfile);
     g_free(line);
+    line=NULL;
     fflush(NULL);
     _exit(123);
 }
@@ -1477,6 +1519,7 @@ static int internal_rw_file(char *target, char *source, off_t size)
 	}
     }
     g_free(buffer);
+    buffer=NULL;
     if(close(source_file) < 0)
     {
 	close(target_file);
