@@ -1,4 +1,4 @@
 gboolean update_dir(GtkTreeView * treeview, GtkTreeRowReference * reference);
 
 void remove_it(GtkTreeView * treeview, GtkTreeRowReference * reference);
-gint local_monitor(gboolean force);
+gint local_monitor(GtkTreeView * treeview,gboolean force);
