@@ -102,7 +102,10 @@ gboolean in_pasteboard(tree_entry_t *en)
 
     if (IS_NETDIR(en->subtype)||IS_NETFILE(en->subtype)){
 	char *server,*remote_file;
-	if (files) g_free(files);
+	if (files) {
+		g_free(files);
+		files=NULL;
+	}
 	server=g_strdup(en->path+2);
 	strtok(server,"/");
 	remote_file=server+strlen(server)+1;
@@ -122,6 +125,7 @@ gboolean in_pasteboard(tree_entry_t *en)
 		  server,remote_file);
 	
 	g_free(server);
+	server=NULL;
 	path=files;
     }
     else path=en->path;
@@ -234,6 +238,7 @@ static void copy_cut(GtkTreeView * treeview, gboolean cut)
 			  (IS_NETDIR(en->subtype))?"/\n":"\n");
 		}
 		g_free(server);
+		server=NULL;
 	        files = files + strlen(files);
 		
 	} else {
@@ -244,6 +249,7 @@ static void copy_cut(GtkTreeView * treeview, gboolean cut)
     /*printf("dbg:len=%d,strlen=%d,data=%s\n",len,strlen(buffer),buffer); */
     XStoreBuffer(GDK_DISPLAY(), buffer, len, CUT_BUFFER);
     g_free(buffer);
+    buffer=NULL;
 
 
     g_list_free(paste_list);
@@ -453,6 +459,7 @@ no_pasteboard:
       tmpfile=CreateSMBTmpList(treeview,list,target,
 		      IS_SAMBA_SERVER(t_en->subtype));
       g_free(target);
+      target=NULL;
       if (!tmpfile) goto paste_done;
       SMBDropFile (treeview,t_en,&iter,tmpfile);
       goto paste_done;
@@ -499,19 +506,30 @@ no_pasteboard:
 	goto paste_done;
     }    
     else if (IS_APP_TYPE(t_en->type)) {
-	char *argv[3];
-	print_status(treeview, "xf_INFO_ICON", 
-			       _("Executing"), " : ", 
-			        FILENAME(t_en), 
-			       NULL);
-	argv[0]=t_en->path;
-	argv[2]=0;
-	
-	for(tmp = list; tmp; tmp = tmp->next)
-	{
+	for(tmp = list; tmp != NULL; tmp = tmp->next)
+	{  
+	    gchar *in_cmd;
+	    
 	    u = tmp->data;
-	    argv[1]=u->url;
-	    runv(treeview, argv);
+	    
+	    if (t_en->filter) {
+		in_cmd = g_strconcat(t_en->path," ",t_en->filter,NULL);
+	    	print_status(treeview, "xf_INFO_ICON", 
+			       _("Executing"), " : ", 
+			        FILENAME(t_en)," ",t_en->filter,
+			       NULL);
+	    }
+	    else {
+		in_cmd = g_strdup(t_en->path);
+	    	print_status(treeview, "xf_INFO_ICON", 
+			       _("Executing"), " : ", 
+			        FILENAME(t_en),
+			       NULL);
+	    }
+	    
+ 	    on_run_path(treeview, in_cmd, u->url, IS_IN_TERM(t_en->subtype),
+			   FALSE, FALSE);
+	    g_free(in_cmd);
 	}
 	goto paste_done;
     }
