@@ -98,7 +98,7 @@ static gboolean cancel_remove;
 static gchar *remote_pass=NULL;
 static gchar *remote_location=NULL;
 static gboolean invalid_input = FALSE;
-
+static int remove_selectionOK=0;
 static gboolean on_destroy_event(GtkWidget * widget, 
 		GdkEvent * event, gpointer user_data);
 static void on_remove_no_activate(GtkButton * button, gpointer user_data);
@@ -143,8 +143,8 @@ static char  *CreateRmTmpList(GtkTreeView *treeview){
 	}
 	fprintf(tmpfile,"del \"%s\";\n",file);
      }
-     g_free(path);
-     g_free(directory);
+     g_free(path);path=NULL;
+     g_free(directory);directory=NULL;
    }
    fclose (tmpfile);
    return fname;
@@ -212,10 +212,10 @@ gboolean unlinkit(GtkTreeView * treeview, char *path)
 
 	    if(!unlinkit(treeview, fullpath))
 	    {
-		g_free(fullpath);
+		g_free(fullpath);fullpath=NULL;
 		return FALSE;
 	    }
-	    g_free(fullpath);
+	    g_free(fullpath);fullpath=NULL;
 	}
 	closedir(directory);
 	if(cancel_remove)
@@ -263,7 +263,7 @@ static void move2trash(GtkTreeView * treeview, char *path)
 		move2trash(treeview, dirlist.gl_pathv[i]);
 	    }
 	}
-	g_free(globstring);
+	g_free(globstring);globstring=NULL;
 	globfree(&dirlist);
     }
     add2trash(treeview, path);
@@ -281,14 +281,14 @@ gboolean wasteit(GtkTreeView * treeview, char *path)
 	return FALSE;
     directory = g_path_get_dirname(path);
     wastepath=g_strconcat(directory,"/..Wastebasket",NULL);
-    g_free(directory);
+    g_free(directory);directory=NULL;
     /*printf("DBG:wastepath=%s\n",wastepath); */
     if(access(wastepath, F_OK) != 0)
     {
 	if(mkdir(wastepath, 0xFFFF) < 0){
 	    print_diagnostics(treeview, "xf_ERROR_ICON",
 			    strerror(errno), "\n", NULL);
-	    g_free(wastepath);
+	    g_free(wastepath);wastepath=NULL;
 	    return FALSE;
 	}
 
@@ -306,17 +306,17 @@ gboolean wasteit(GtkTreeView * treeview, char *path)
 	if(rename(wastename, wastebackup) < 0){
 	    print_diagnostics(treeview, "xf_ERROR_ICON",
 			    strerror(errno), "\n", NULL);
-	    g_free(wastebackup); 
-	    g_free(wastename);
-	    g_free(wastepath);
+	    g_free(wastebackup); wastebackup=NULL;
+	    g_free(wastename);wastename=NULL;
+	    g_free(wastepath);wastepath=NULL;
 	    return FALSE;
 	}
 	if(rename(path, wastename) < 0){
 	    print_diagnostics(treeview, "xf_ERROR_ICON",
 			    strerror(errno), "\n", NULL);
-	    g_free(wastebackup); 
-	    g_free(wastename);
-	    g_free(wastepath);
+	    g_free(wastebackup); wastebackup=NULL;
+	    g_free(wastename);wastename=NULL;
+	    g_free(wastepath);wastepath=NULL;
 	    return FALSE;
 	}
 	/* ENABLE_AUTO_T is really to disable */
@@ -328,15 +328,15 @@ gboolean wasteit(GtkTreeView * treeview, char *path)
 	if(rename(path, wastename) < 0){
 	    print_diagnostics(treeview, "xf_ERROR_ICON",
 			    strerror(errno), "\n", NULL);
-	    g_free(wastename);
-	    g_free(wastepath);
+	    g_free(wastename);wastename=NULL;
+	    g_free(wastepath);wastepath=NULL;
 	    return FALSE;
 	}
 	if(!(tree_details->preferences & ENABLE_AUTO_T))
 	    move2trash(treeview, wastename);
     }
-    g_free(wastename);
-    g_free(wastepath);
+    g_free(wastename);wastename=NULL;
+    g_free(wastepath);wastepath=NULL;
     return TRUE;
 }
 
@@ -368,17 +368,17 @@ static void check_select(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
    	   if (!remote_location)remote_location=location;
 	   else {
 	     if (strcmp(remote_location,location)!=0)invalid_input=TRUE;
-	     g_free(location);
+	     g_free(location);location=NULL;
 	   }
    	   if (!remote_pass)remote_pass=server_pass;
 	   else {
 	     if (strcmp(remote_pass,server_pass)!=0)invalid_input=TRUE;
-	     g_free(server_pass);
+	     g_free(server_pass);server_pass=NULL;
 	   }
 	}
     }
     if (IS_PATH(en->type)&&remote_location) invalid_input=TRUE;
-    tree_details->selectionOK++;
+    remove_selectionOK++;
     reference = gtk_tree_row_reference_new(treemodel, path);
     remove_list = g_list_append(remove_list, reference);    
     return;
@@ -475,14 +475,14 @@ static void make_dialog(GtkTreeView * treeview, gboolean question)
 			    "(",time_to_string(en->st->st_mtime)," ",
 			    sizetag((long long)en->st->st_size, -1),")",
 			    NULL); 
-	g_free(t);
+	g_free(t);t=NULL;
     }
 
     gtk_label_set_text((GtkLabel *) 
 		    lookup_widget(tree_details->remove, "label20"), 
 		    mess);
-    g_free(mess);
-    g_free(mess1);
+    g_free(mess);mess=NULL;
+    g_free(mess1);mess1=NULL;
     gtk_widget_realize(tree_details->remove);
     /*gdk_window_set_decorations (
        tree_details->remove->window,
@@ -506,7 +506,7 @@ static void do_remove(GtkWidget * widget)
     
     if(!set_load_wait(&tree_details))
 	return;
-    tree_details->selectionOK = FALSE;
+    remove_selectionOK = FALSE;
     all = FALSE;
     unlink_mode = FALSE;
     invalid_input = FALSE;
@@ -514,10 +514,10 @@ static void do_remove(GtkWidget * widget)
 		    check_select, (gpointer) treeview);
     if (invalid_input){
 	    remove_list=clear_remove_list(remove_list);
-	    tree_details->selectionOK=0;
+	    remove_selectionOK=0;
     }
     unset_load_wait(&tree_details);
-    if(!tree_details->selectionOK)
+    if(!remove_selectionOK)
     {
 	print_status(treeview, "xf_WARNING_ICON,",strerror(EINVAL),NULL);
 	return;
@@ -588,6 +588,8 @@ static void on_remove_no_activate(GtkButton * button, gpointer user_data)
 	goto loop;
     if(remove_list)
 	make_dialog(treeview, TRUE);
+    turn_on(treeview);
+    
 
 }
 
@@ -676,6 +678,7 @@ static void on_remove_yes_activate(GtkButton * button, gpointer user_data)
 	unset_load_wait(&tree_details);
 	remove_list = g_list_remove(remove_list, remove_list->data);
 	hide_stop(tree_details->window);
+        turn_on(treeview);
 	return;
     }
 
@@ -699,6 +702,7 @@ static void on_remove_yes_activate(GtkButton * button, gpointer user_data)
     {
 	unset_load_wait(&tree_details);
 	make_dialog(treeview, FALSE);
+        turn_on(treeview);
 	return;
     }
 
@@ -746,6 +750,7 @@ static void on_remove_yes_activate(GtkButton * button, gpointer user_data)
     }
     
     remote_remove_list=clear_remove_list(remote_remove_list);
+    turn_on(treeview);
 
     return;
 }
@@ -846,8 +851,8 @@ static void make_single_rm_dialog(GtkTreeView * treeview,
     gtk_label_set_text((GtkLabel *) 
 		    lookup_widget(tree_details->remove,"label20"), 
 		    mess);
-    g_free(mess);
-    g_free(mess1);
+    g_free(mess);mess=NULL;
+    g_free(mess1);mess1=NULL;
     gtk_widget_realize(tree_details->remove);
     gtk_widget_show(tree_details->remove);
 
