@@ -62,12 +62,12 @@ static void new_terminal(GtkTreeView * treeview)
 			term = NULL;
 			printf("DBG:not valid %s\n",getenv("TERM"));
 		}
-		g_free(c);
-		g_free(t);
+		g_free(c);c=NULL;
+		g_free(t);t=NULL;
     }
     if (!term) term=g_strdup("xterm +mesg -aw +cm +dc -j -rightbar -s -sb -u8 -wf -sl 1024");
     on_run(treeview,term,NULL, FALSE, FALSE, FALSE);
-    g_free(term);
+    g_free(term);term=NULL;
     return;
 }
 
