@@ -1,4 +1,4 @@
 int open_trash(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data);
 int collect_trash(GtkTreeView * treeview, char *path);
-int add2trash(char *path);
+int add2trash(GtkTreeView * treeview, char *path);
 int count_trash(void);
