@@ -1,15 +1,7 @@
 
-#ifndef __XFFM_TYPES_H__
-#define __XFFM_TYPES_H__
+#ifndef __TYPES_H__
+#define __TYPES_H__
 #include <time.h>
-/* only 2 treeview supported at present time */
-enum
-{
-    RIGHT_TREEVIEW,
-    LEFT_TREEVIEW,
-    TREECOUNT
-};
-
 enum
 {
     OTHER_INPUT,
@@ -18,8 +10,6 @@ enum
     RUN_DOUBLE_CLICK,
     PRINT_INPUT,
     NEW_INPUT,
-    NEW_FILE_INPUT,
-    NEW_DIR_INPUT,
     RENAME_INPUT,
     DUPLICATE_INPUT,
     SYMLINK_INPUT,
@@ -28,14 +18,19 @@ enum
     WORKDIR_INPUT,
     OPEN_BOOK_INPUT,
     SAVE_BOOK_INPUT,
-    NEW_BOOK_INPUT,
-    NEW_WIN_INPUT,
-    NEW_BOOK_WIN_INPUT,
-    NEW_TERM_INPUT,
-    FREQUENCY_INPUT,
-    RECENT_INPUT
+    NEW_BOOK_INPUT
+	    
 };
 
+typedef struct history_combo_info_t{
+	GtkTreeView 	*treeview;
+	GtkWidget 	*combo;
+	GtkWidget 	*entry;
+	GList 		*list;
+	GList 		*limited_list;
+	gchar 		*active_dbh_file;
+}history_combo_info_t;
+
 typedef struct preferences_t {
   char *label;
   char *xml_option;
@@ -51,72 +46,49 @@ typedef struct golist
 }
 golist;
 
+enum
+{
+    SMALL,
+    MEDIUM,
+    REAL_BIG,
+    BIG,			/* must be last one */
+    HOW_MANY_SIZES
+};
 
 enum
 {
-    EMPTY_COLUMN,
-    ENTRY_COLUMN,
-    STYLE_COLUMN,
-    FONT_COLUMN,
-    SFONT_COLUMN,
-    EDITABLE_COLUMN,
-    COLOUR_COLUMN,
-    WEIGHT_COLUMN,
-
-    PIXBUF_COLUMN,
-    NAME_COLUMN,
-    SIZE_COLUMN,
-    DATE_COLUMN,
-    OWNER_COLUMN,
-    GROUP_COLUMN,
-    MODE_COLUMN,
-
-    TREE_COLUMNS
+    SIZE_COL,
+    DATE_COL,
+    UID_COL,
+    GID_COL,
+    MODE_COL,
+    NUM_COLS
 };
 
-typedef struct treestuff_t {
-    GtkTreeView *treeview;
-    GtkTreeModel *treemodel;
-    GtkTreeSortable *sortable;
-    GtkTreeSelection *selection;
-
-    GtkTreeViewColumn *column[TREE_COLUMNS];
-    unsigned preferences;
-    golist *gogo;
-    gint ascending;
-    gint sort_column;
-    gint name_size;
-}treestuff_t;
 
 typedef struct tree_details_t
 {
-    /* widgets */
-    GtkWidget *window;
-    GtkWidget *remove;
-    /* global options */
-    unsigned preferences;
-    gchar *theme;
-    unsigned recent_days;
-    gint frequent_count;
-    gchar *argv[2];
-    /* geometric stuff */
-    gint  geometryX;
-    gint  geometryY;
-    gint  hpane;
-    double hpane_ratio;
-    /* smart button status */
-    GHashTable *smart_hash;
-    /* individual treeview information */
-    treestuff_t treestuff[TREECOUNT];
-    /* process variables */
     void *tubo_object;
-    gint selectionOK;
-    gint timer;
-    gint input;
-    gboolean loading;
-    gboolean stop;
+    int selectionOK;
+    GtkWidget *window;
+    GtkTreeView *treeview;
+    GtkTextView *status;
+    GtkTextView *diagnostics;
+    golist *gogo;
+    int input;
+    int timer;
+    int loading;
+    GtkTreeViewColumn *column[NUM_COLS];
+    GtkWidget *remove;
+
+    unsigned preferences;
+    char *homedir;
+    char *smb_user;
     int icon_size;
-    gchar *sidebar;
+    int xffm_layer;
+    char *theme;
+    /*GHashTable *file_hash; */
+    /*GtkTextBuffer *text_buffer; */
 }
 tree_details_t;
 
@@ -130,7 +102,7 @@ typedef struct tree_entry_t
     char *tag;
     char *path;			/* this is just name for trash */
     char *filter;		/* filter for directories, path for trash */
-    off_t checksum;		/* only for directories */
+    long long checksum;		/* only for directories */
     time_t load_time;
 }
 tree_entry_t;
@@ -144,12 +116,12 @@ typedef struct dir_t
 dir_t;
 
 
-typedef struct xfdir_t
+typedef struct gdir_t
 {
     size_t pathc;		/* Count of paths matched */
     dir_t *gl;
 }
-xfdir_t;
+gdir_t;
 
 typedef struct column_info_t
 {
@@ -158,7 +130,73 @@ typedef struct column_info_t
 }
 column_info_t;
 
+#if 0
+enum
+{
+    /* gtk stock icons first */
+    FIND_RESULT_ICON,
+    ERROR_ICON,
+    INFO_ICON,
+    QUESTION_ICON,
+    WARNING_ICON,
+    FIND_ICON,
+    BOOKMARKS_ICON,
+    TASKS_ICON,
+    TRASH_CLOSED_ICON,
+    TRASH_OPEN_ICON,
+    UP_ICON,
 
+    WASTE_BASKET_EMPTY_ICON,
+    WASTE_BASKET_FULL_ICON,
+    /* application icons */
+    DEFAULT_ICON,
+    OPEN_FOLDER_ICON,
+    OPEN_FOLDER_LNK_ICON,
+    OPEN_FOLDER_RO_ICON,
+    OPEN_FOLDER_RO_LNK_ICON,
+    HOPEN_FOLDER_ICON,
+    HOPEN_FOLDER_LNK_ICON,
+    HOPEN_FOLDER_RO_ICON,
+    HOPEN_FOLDER_RO_LNK_ICON,
+    AOPEN_FOLDER_ICON,
+    AOPEN_FOLDER_LNK_ICON,
+    AOPEN_FOLDER_RO_ICON,
+    AOPEN_FOLDER_RO_LNK_ICON,
+
+    CLOSED_FOLDER_ICON,
+    CLOSED_FOLDER_LNK_ICON,
+    CLOSED_FOLDER_RO_ICON,
+    CLOSED_FOLDER_RO_LNK_ICON,
+    NOACCESS_ICON,
+    LINK_ICON,
+    BROKEN_ICON,
+    EXECUTABLE_ICON,
+    NETWORK_ICON,
+    BLOCK_DEV_ICON,
+    CHAR_DEV_ICON,
+    FIFO_ICON,
+    SOCKET_ICON,
+    HOW_MANY_ICONS
+};
+#endif
+
+enum
+{
+    EMPTY_COLUMN,
+    ENTRY_COLUMN,
+    STYLE_COLUMN,
+    FONT_COLUMN,
+
+    PIXBUF_COLUMN,
+    NAME_COLUMN,
+    SIZE_COLUMN,
+    DATE_COLUMN,
+    OWNER_COLUMN,
+    GROUP_COLUMN,
+    MODE_COLUMN,
+
+    TREE_COLUMNS
+};
 
 typedef struct root_t
 {
@@ -174,13 +212,10 @@ enum
     /*ROOT_TASKS,*/
     ROOT_BOOKMARKS,
     ROOT_FILES,
-#ifdef USE_SMB_BRANCH
     ROOT_NETWORK,
-#endif
+    ROOT_APP,
     ROOT_FIND,
     ROOT_TRASH,
-    ROOT_RECENT,
-    ROOT_FREQUENT,
 #if defined(HAVE_GETMNTENT) || defined(HAVE_GETFSENT) || defined(HAVE_GETVFSENT)
     ROOT_FSTAB,
 #endif
