@@ -105,6 +105,7 @@ enum {
     XFFM_HIDE_TRASH,
     XFFM_HIDE_FSTAB,
     XFFM_DEFAULT_UNLINK,
+    XFFM_USE_SUDO,
     XFFM_OPTIONS
 };
 
@@ -131,6 +132,7 @@ static Options xffm_options[XFFM_OPTIONS]={
 	{ "XFFM_HIDE_FIND",          NULL },
 	{ "XFFM_HIDE_TRASH",         NULL },
 	{ "XFFM_HIDE_FSTAB",         NULL },
+	{ "XFFM_USE_SUDO",         NULL },
 	{ "XFFM_DEFAULT_UNLINK",     NULL }
 };
 
@@ -168,11 +170,12 @@ static void create_channel(void)
 {
     McsSetting *setting;
 
-    const gchar *home = g_get_home_dir();
+    /*const gchar *home = g_get_home_dir();*/
     gchar *rcfile;
     gint i;
 
-    rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);
+    /*rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);*/
+    rcfile = xfce_get_userfile(RCDIR, RCFILE, NULL);
     mcs_manager_add_channel_from_file(mcs_manager, CHANNEL, rcfile);
     g_free(rcfile);rcfile=NULL;
 
@@ -198,11 +201,12 @@ static void create_channel(void)
 
 static gboolean write_options(void)
 {
-    const gchar *home = g_get_home_dir();
+    /*const gchar *home = g_get_home_dir();*/
     gchar *rcfile;
     gboolean result;
 
-    rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);
+    rcfile = xfce_get_userfile(RCDIR, RCFILE, NULL);
+/*    rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);*/
     result = mcs_manager_save_channel_to_file(mcs_manager, CHANNEL, rcfile);
     g_free(rcfile);
     rcfile=NULL;
