@@ -137,25 +137,27 @@ char *sizetag(long long tama, int count)
 {
     char *tag = "B";
     static char buf[64];
+    unsigned long long utama=(unsigned long long)tama;
 
     buf[0] = 0;
     if(tama >= 0)
     {
-	if(tama >= (unsigned long long)1024 * 1024 * 1024)
+	if(utama >= 1024 * 1024 * 1024)
 	{
-	    (unsigned long long)tama /= (unsigned long long)1024 *1024 * 1024;
+	    utama /= (unsigned long long)1024 *1024 * 1024;
 	    tag = "GB";
 	}
-	else if(tama >= 1024 * 1024)
+	else if(utama >= 1024 * 1024)
 	{
-	    (unsigned long long)tama /= 1024 * 1024;
+	    utama /= 1024 * 1024;
 	    tag = "MB";
 	}
-	else if(tama >= 1024)
+	else if(utama >= 1024)
 	{
-	    (unsigned long long)tama /= 1024;
+	    utama /= 1024;
 	    tag = "KB";
 	}
+	tama=utama;
 	if(count > 0)
 	    sprintf(buf, "%d %s, %llu %s.", count, (count == 1) ? _("file") : _("files"), tama, tag);
 	else
@@ -224,7 +226,6 @@ void update_columns(GtkTreeModel * treemodel, GtkTreeIter * target, tree_entry_t
     struct group *g;
     struct passwd *p;
     char *group, *owner, *tag;
-    char date[64];
 
     if(IS_DIR(en->type))
 	tag = sizetag(-1, en->count);
