@@ -105,13 +105,17 @@ static gchar *my_utf_string(char *t)
     
     /*printf("DBG: codeset for system is: %s\n",from_codeset);    */
     
-    if(s) g_free(s);
+    if(s) {
+	    g_free(s);
+	    s=NULL;
+    }
     /*printf("DBG: utfing %s\n",t); */
     for(c = (unsigned char *)t; *c != 0; c++)
 	if(*c < 32 && *c != '\n')
 	    *c = ' ';
     s = g_convert (t,strlen(t),"UTF-8",from_codeset,&r_bytes, &w_bytes, &error);
     g_free(from_codeset);
+    from_codeset=NULL;
 
     if(!s)
     {
