@@ -705,7 +705,7 @@ cb_do_patch (GtkWidget * widget, gpointer data)
     xf_dlg_info (diff,texto);
     free (texto);
     /*  */
-    g_free (s);
+    g_free (s);s=NULL;
     return;
   }
   if (patching)
@@ -719,7 +719,7 @@ cb_do_patch (GtkWidget * widget, gpointer data)
   first_diff ();
   update_titlesP ();
   /*  */
-  g_free (s);
+  g_free (s);s=NULL;
   return;
 }
 
